@@ -14,7 +14,12 @@ module RSpec
     def self.define_optimized_require_for_rspec(lib, &require_relative)
       name = "require_rspec_#{lib}"
 
-      if Kernel.respond_to?(:require_relative)
+      if RUBY_PLATFORM == 'java' && !Kernel.respond_to?(:require)
+        # JRuby 9.1.17.0 has developed a regression for require
+        (class << self; self; end).__send__(:define_method, name) do |f|
+          Kernel.send(:require, "rspec/#{lib}/#{f}")
+        end
+      elsif Kernel.respond_to?(:require_relative)
         (class << self; self; end).__send__(:define_method, name) do |f|
           require_relative.call("#{lib}/#{f}")
         end
@@ -73,9 +78,25 @@ module RSpec
       end
     end
 
+    # @api private
+    #
+    # Used internally to get a class of a given object, even if it does not respond to #class.
+    def self.class_of(object)
+      object.class
+    rescue NoMethodError
+      singleton_class = class << object; self; end
+      singleton_class.ancestors.find { |ancestor| !ancestor.equal?(singleton_class) }
+    end
+
     # A single thread local variable so we don't excessively pollute that namespace.
-    def self.thread_local_data
-      Thread.current[:__rspec] ||= {}
+    if RUBY_VERSION.to_f >= 2
+      def self.thread_local_data
+        Thread.current.thread_variable_get(:__rspec) || Thread.current.thread_variable_set(:__rspec, {})
+      end
+    else
+      def self.thread_local_data
+        Thread.current[:__rspec] ||= {}
+      end
     end
 
     # @api private
@@ -129,7 +150,7 @@ module RSpec
       end
     end
 
-    # The Differ is only needed when a a spec fails with a diffable failure.
+    # The Differ is only needed when a spec fails with a diffable failure.
     # In the more common case of all specs passing or the only failures being
     # non-diffable, we can avoid the extra cost of loading the differ, diff-lcs,
     # pp, etc by avoiding an unnecessary require. Instead, autoload will take
