@@ -7,7 +7,7 @@ module RSpec
     #
     # Provides query methods for different OS or OS features.
     module OS
-      module_function
+    module_function
 
       def windows?
         !!(RbConfig::CONFIG['host_os'] =~ /cygwin|mswin|mingw|bccwin|wince|emx/)
@@ -22,7 +22,7 @@ module RSpec
     #
     # Provides query methods for different rubies
     module Ruby
-      module_function
+    module_function
 
       def jruby?
         RUBY_PLATFORM == 'java'
@@ -47,6 +47,10 @@ module RSpec
       def mri?
         !defined?(RUBY_ENGINE) || RUBY_ENGINE == 'ruby'
       end
+
+      def truffleruby?
+        defined?(RUBY_ENGINE) && RUBY_ENGINE == 'truffleruby'
+      end
     end
 
     # @api private
@@ -54,7 +58,23 @@ module RSpec
     # Provides query methods for ruby features that differ among
     # implementations.
     module RubyFeatures
-      module_function
+    module_function
+
+      if Ruby.jruby? && RUBY_VERSION.to_f < 1.9
+        # On JRuby 1.7 `--1.8` mode, `Process.respond_to?(:fork)` returns true,
+        # but when you try to fork, it raises an error:
+        #   NotImplementedError: fork is not available on this platform
+        #
+        # When we drop support for JRuby 1.7 and/or Ruby 1.8, we can drop
+        # this special case.
+        def fork_supported?
+          false
+        end
+      else
+        def fork_supported?
+          Process.respond_to?(:fork)
+        end
+      end
 
       def optional_and_splat_args_supported?
         Method.method_defined?(:parameters)
@@ -74,17 +94,31 @@ module RSpec
         end
       end
 
+      if RUBY_VERSION.to_f >= 2.7
+        def supports_taint?
+          false
+        end
+      else
+        def supports_taint?
+          true
+        end
+      end
       ripper_requirements = [ComparableVersion.new(RUBY_VERSION) >= '1.9.2']
 
       ripper_requirements.push(false) if Ruby.rbx?
 
       if Ruby.jruby?
         ripper_requirements.push(Ruby.jruby_version >= '1.7.5')
-        # Ripper on JRuby 9.0.0.0.rc1 or later reports wrong line number
+        # Ripper on JRuby 9.0.0.0.rc1 - 9.1.8.0 reports wrong line number
         # or cannot parse source including `:if`.
-        ripper_requirements.push(Ruby.jruby_version < '9.0.0.0.rc1')
+        # Ripper on JRuby 9.x.x.x < 9.1.17.0 can't handle keyword arguments
+        # Neither can JRuby 9.2, e.g. < 9.2.1.0
+        ripper_requirements.push(!Ruby.jruby_version.between?('9.0.0.0.rc1', '9.2.0.0'))
       end
 
+      # TruffleRuby disables ripper due to low performance
+      ripper_requirements.push(false) if Ruby.truffleruby?
+
       if ripper_requirements.all?
         def ripper_supported?
           true
@@ -95,6 +129,10 @@ module RSpec
         end
       end
 
+      def distincts_kw_args_from_positional_hash?
+        RUBY_VERSION >= '3.0.0'
+      end
+
       if Ruby.mri?
         def kw_args_supported?
           RUBY_VERSION >= '2.0.0'
@@ -109,7 +147,6 @@ module RSpec
         end
       else
         # RBX / JRuby et al support is unknown for keyword arguments
-        # rubocop:disable Lint/Eval
         begin
           eval("o = Object.new; def o.m(a: 1); end;"\
                " raise SyntaxError unless o.method(:m).parameters.include?([:key, :a])")
@@ -147,7 +184,6 @@ module RSpec
             false
           end
         end
-        # rubocop:enable Lint/Eval
       end
 
       def module_refinement_supported?
