@@ -77,6 +77,9 @@ module RSpec
           # This is a JRuby 9.2.1.x error
           %r{jruby/kernel/gem_prelude},
           %r{lib/jruby\.jar!/jruby/preludes},
+          # Ignore some JRuby errors for gems
+          %r{jruby/\d\.\d(\.\d)?/gems/aruba},
+          %r{jruby/\d\.\d(\.\d)?/gems/ffi},
         ]
 
       def strip_known_warnings(input)
